@@ -8,7 +8,9 @@ use Capture::Tiny 'capture';
 
 use Data::Section::Simple 'get_data_section';
 
-use File::Which; # For which().
+use File::Basename;	# For fileparse().
+use File::Temp;		# For newdir().
+use File::Which;	# For which().
 
 use Moo;
 
@@ -18,11 +20,13 @@ use Set::Array;
 
 use Try::Tiny;
 
+use Types::Standard qw/Any HashRef Int Str/;
+
 has command =>
 (
 	default  => sub{return Set::Array -> new},
 	is       => 'rw',
-#	isa      => 'Set::Array',
+	isa      => Any,
 	required => 0,
 );
 
@@ -30,7 +34,7 @@ has dot_input =>
 (
 	default  => sub{return ''},
 	is       => 'rw',
-#	isa      => 'Str',
+	isa      => Str,
 	required => 0,
 );
 
@@ -38,7 +42,7 @@ has dot_output =>
 (
 	default  => sub{return ''},
 	is       => 'rw',
-#	isa      => 'Str',
+	isa      => Str,
 	required => 0,
 );
 
@@ -46,7 +50,7 @@ has edge =>
 (
 	default  => sub{return {} },
 	is       => 'rw',
-#	isa      => 'HashRef',
+	isa      => HashRef,
 	required => 0,
 );
 
@@ -54,7 +58,7 @@ has edge_hash =>
 (
 	default  => sub{return {} },
 	is       => 'rw',
-#	isa      => 'HashRef',
+	isa      => HashRef,
 	required => 0,
 );
 
@@ -62,7 +66,7 @@ has global =>
 (
 	default  => sub{return {} },
 	is       => 'rw',
-#	isa      => 'HashRef',
+	isa      => HashRef,
 	required => 0,
 );
 
@@ -70,7 +74,15 @@ has graph =>
 (
 	default  => sub{return {} },
 	is       => 'rw',
-#	isa      => 'HashRef',
+	isa      => HashRef,
+	required => 0,
+);
+
+has im_meta =>
+(
+	default  => sub{return {} },
+	is       => 'rw',
+	isa      => HashRef,
 	required => 0,
 );
 
@@ -78,7 +90,7 @@ has logger =>
 (
 	default  => sub{return ''},
 	is       => 'rw',
-#	isa      => 'Str',
+	isa      => Any,
 	required => 0,
 );
 
@@ -86,7 +98,7 @@ has node =>
 (
 	default  => sub{return {} },
 	is       => 'rw',
-#	isa      => 'HashRef',
+	isa      => HashRef,
 	required => 0,
 );
 
@@ -94,7 +106,7 @@ has node_hash =>
 (
 	default  => sub{return {} },
 	is       => 'rw',
-#	isa      => 'HashRef',
+	isa      => HashRef,
 	required => 0,
 );
 
@@ -102,7 +114,7 @@ has scope =>
 (
 	default  => sub{return Set::Array -> new},
 	is       => 'rw',
-#	isa      => 'Set::Array',
+	isa      => Any,
 	required => 0,
 );
 
@@ -110,7 +122,7 @@ has subgraph =>
 (
 	default  => sub{return {} },
 	is       => 'rw',
-#	isa      => 'HashRef',
+	isa      => HashRef,
 	required => 0,
 );
 
@@ -118,7 +130,7 @@ has verbose =>
 (
 	default  => sub{return 0},
 	is       => 'rw',
-#	isa      => 'Int',
+	isa      => Int,
 	required => 0,
 );
 
@@ -126,39 +138,47 @@ has valid_attributes =>
 (
 	default  => sub{return {} },
 	is       => 'rw',
-#	isa      => 'HashRef',
+	isa      => HashRef,
 	required => 0,
 );
 
-our $VERSION = '2.42';
+our $VERSION = '2.43';
 
 # -----------------------------------------------
 
 sub BUILD
 {
 	my($self)    = @_;
-	my($options) = $self -> global;
+	my($globals) = $self -> global;
 	my($dot)     = which('dot');
 	my($global)  =
 	{
-		directed     => $$options{directed}      ? 'digraph'         : 'graph',
-		driver       => $$options{driver}        ? $$options{driver} : $dot,
-		format       => $$options{format}        ? $$options{format} : 'svg',
-		label        => $$options{directed}      ? '->'              : '--',
-		name         => defined($$options{name}) ? $$options{name}   : 'Perl',
-		record_shape => ($$options{record_shape} && $$options{record_shape} =~ /^(M?record)$/) ? $1 : 'Mrecord',
-		strict       => defined($$options{strict})  ? $$options{strict}   :  0,
-		subgraph     => $$options{subgraph}         ? $$options{subgraph} : {},
-		timeout      => defined($$options{timeout}) ? $$options{timeout}  : 10,
+		directed		=> $$globals{directed}			? 'digraph'				: 'graph',
+		driver			=> $$globals{driver}			? $$globals{driver}		: $dot,
+		format			=> $$globals{format}			? $$globals{format}		: 'svg',
+		im_format		=> $$globals{im_format}			? $$globals{im_format}	: 'cmapx',
+		label			=> $$globals{directed}			? '->'					: '--',
+		name			=> defined($$globals{name})		? $$globals{name}		: 'Perl',
+		record_shape	=> ($$globals{record_shape} && $$globals{record_shape} =~ /^(M?record)$/) ? $1 : 'Mrecord',
+		strict			=> defined($$globals{strict})	? $$globals{strict}		:  0,
+		subgraph		=> $$globals{subgraph}			? $$globals{subgraph}	: {},
+		timeout			=> defined($$globals{timeout})	? $$globals{timeout}	: 10,
+	};
+	my($im_metas)	= $self -> im_meta;
+	my($im_meta)	=
+	{
+		URL => $$im_metas{URL} ? $$im_metas{URL} : '',
 	};
 
 	$self -> global($global);
+	$self -> im_meta($im_meta);
 	$self -> load_valid_attributes;
-	$self -> validate_params('global',   %{$self -> global});
-	$self -> validate_params('graph',    %{$self -> graph});
-	$self -> validate_params('node',     %{$self -> node});
-	$self -> validate_params('edge',     %{$self -> edge});
-	$self -> validate_params('subgraph', %{$self -> subgraph});
+	$self -> validate_params('global',		%{$self -> global});
+	$self -> validate_params('graph',		%{$self -> graph});
+	$self -> validate_params('im_meta',		%{$self -> im_meta});
+	$self -> validate_params('node',		%{$self -> node});
+	$self -> validate_params('edge',		%{$self -> edge});
+	$self -> validate_params('subgraph',	%{$self -> subgraph});
 	$self -> scope -> push
 		({
 			edge     => $self -> edge,
@@ -167,15 +187,22 @@ sub BUILD
 			subgraph => $self -> subgraph,
 		 });
 
-	my(%global) = %{$self -> global};
+	my(%global)		= %{$self -> global};
+	my(%im_meta)	= %{$self -> im_meta};
 
-	$self -> log(debug => "Default global: $_ => $global{$_}") for sort keys %global;
+	$self -> log(debug => "Default global:  $_ => $global{$_}")	for sort keys %global;
+	$self -> log(debug => "Default im_meta: $_ => $im_meta{$_}")	for grep{$im_meta{$_} } sort keys %im_meta;
 
 	my($command) = (${$self -> global}{strict} ? 'strict ' : '')
 		. (${$self -> global}{directed} . ' ')
 		. ${$self -> global}{name}
 		. "\n{\n";
 
+	for my $key (grep{$im_meta{$_} } sort keys %im_meta)
+	{
+		$command .= qq|$key = "$im_meta{$key}"; \n|;
+	}
+
 	$self -> command -> push($command);
 
 	$self -> default_graph;
@@ -555,9 +582,9 @@ sub load_valid_attributes
 	# Since V 2.24, output formats are no longer read from the __DATA__ section.
 	# Rather, they are extracted from the stderr output of 'dot -T?'.
 
-	my($stdout, $stderr)          = capture{system 'dot', '-T?'};
-	my(@field)                    = split(/one of:\s+/, $stderr);
-	$attribute{output_format}{$_} = 1 for split(/\s+/, $field[1]);
+	my($stdout, $stderr)			= capture{system 'dot', '-T?'};
+	my(@field)						= split(/one of:\s+/, $stderr);
+	$attribute{output_format}{$_}	= 1 for split(/\s+/, $field[1]);
 
 	$self -> valid_attributes(\%attribute);
 
@@ -723,17 +750,84 @@ sub report_valid_attributes
 
 sub run
 {
-	my($self, %arg)  = @_;
-	my($driver)      = delete $arg{driver}      || ${$self -> global}{driver};
-	my($format)      = delete $arg{format}      || ${$self -> global}{format};
-	my($timeout)     = delete $arg{timeout}     || ${$self -> global}{timeout};
-	my($output_file) = delete $arg{output_file} || '';
-	my($prefix)      = $format;
-	$prefix          =~ s/:.+$//;
-	%arg             = ($prefix => 1);
+	my($self, %arg)		= @_;
+	my($driver)			= delete $arg{driver}			|| ${$self -> global}{driver};
+	my($format)			= delete $arg{format}			|| ${$self -> global}{format};
+	my($im_format)		= delete $arg{im_format}		|| ${$self -> global}{im_format};
+	my($timeout)		= delete $arg{timeout}			|| ${$self -> global}{timeout};
+	my($output_file)	= delete $arg{output_file}		|| '';
+	my($im_output_file)	= delete $arg{im_output_file}	|| '';
+	my($prefix)			= $format;
+	$prefix				=~ s/:.+$//; # In case of 'png:gd', etc.
+	%arg				= ($prefix => 1);
 
 	$self -> validate_params('output_format', %arg);
 
+	my($prefix_1)	= $im_format;
+	$prefix_1		=~ s/:.+$//; # In case of 'png:gd', etc.
+	%arg			= ($prefix_1 => 1);
+
+	$self -> validate_params('output_format', %arg);
+
+	# Warning: Do not use $im_format in this 'if', because it has a default value.
+
+	if ($im_output_file)
+	{
+		return $self -> run_map($driver, $output_file, $format, $timeout, $im_output_file, $im_format);
+	}
+	else
+	{
+		return $self -> run_mapless($driver, $output_file, $format, $timeout);
+	}
+
+} # End of run.
+
+# -----------------------------------------------
+
+sub run_map
+{
+	my($self, $driver, $output_file, $format, $timeout, $im_output_file, $im_format) = @_;
+
+	$self -> log(debug => "Driver: $driver. Output file: $output_file. Format: $format. IM output file: $im_output_file. IM format: $im_format. Timeout: $timeout second(s)");
+	$self -> log;
+
+	my($result);
+
+	try
+	{
+		$self -> dot_input(join('', @{$self -> command -> print}) . "}\n");
+		$self -> log(debug => $self -> dot_input);
+
+		# The EXLOCK option is for BSD-based systems.
+
+		my($temp_dir)	= File::Temp -> newdir('temp.XXXX', CLEANUP => 1, EXLOCK => 0, TMPDIR => 1);
+		my($temp_file)	= File::Spec -> catfile($temp_dir, "$output_file.gv");
+
+		open(my $fh, '> :raw', $temp_file) || die "Can't open(> $temp_file): $!";
+		print $fh $self -> dot_input;
+		close $fh;
+
+		my(@args) = ("-T$im_format", "-o$im_output_file", "-T$format", "-o$output_file", $temp_file);
+
+		system($driver, @args);
+	}
+	catch
+	{
+		$result = $_;
+	};
+
+	die $result if ($result);
+
+	return $self;
+
+} # End of run_map.
+
+# -----------------------------------------------
+
+sub run_mapless
+{
+	my($self, $driver, $output_file, $format, $timeout) = @_;
+
 	$self -> log(debug => "Driver: $driver. Output file: $output_file. Format: $format. Timeout: $timeout second(s)");
 	$self -> log;
 
@@ -749,6 +843,7 @@ sub run
 		# Usage of utf8 here relies on ISO-8859-1 matching Unicode for low chars.
 		# It saves me the effort of determining if the input contains Unicode.
 
+
 		run3
 			[$driver, "-T$format"],
 			\$self -> dot_input,
@@ -782,7 +877,7 @@ sub run
 
 	return $self;
 
-} # End of run.
+} # End of run_mapless.
 
 # -----------------------------------------------
 
@@ -857,6 +952,8 @@ Or, hit L<the demo page|http://savage.net.au/Perl-modules/html/graphviz2/index.h
 
 =head2 Perl code
 
+=head3 Typical Usage
+
 	#!/usr/bin/env perl
 
 	use strict;
@@ -927,6 +1024,12 @@ Or, hit L<the demo page|http://savage.net.au/Perl-modules/html/graphviz2/index.h
 
 This program ships as scripts/sub.graph.pl. See L</Scripts Shipped with this Module>.
 
+=head3 Image Maps Usage
+
+As of V 2.43, C<GraphViz2> supports image maps, both client and server side.
+
+See L</Image Maps> below.
+
 =head1 Description
 
 =head2 Overview
@@ -947,6 +1050,10 @@ while extending support to all the latest features of L<Graphviz|http://www.grap
 To ensure L<GraphViz2> is a light-weight module, L<Moo> has been used to provide getters and setters,
 rather than L<Moose>.
 
+As of V 2.43, C<GraphViz2> supports image maps, both client and server side.
+
+See L</Image Maps> below.
+
 =head2 What is a Graph?
 
 An undirected graph is a collection of nodes optionally linked together with edges.
@@ -1281,6 +1388,128 @@ chosen by cascading thru a set of options.
 I've posted an email to the L<Graphviz|http://www.graphviz.org/> mailing list suggesting a new option, framecolor, so deal with
 this issue, including a special color of 'invisible'.
 
+=head1 Image Maps
+
+As of V 2.43, C<GraphViz2> supports image maps, both client and server side.
+
+=head2 The Default URL
+
+See the L<Graphviz docs for 'cmapx'|http://www.graphviz.org/doc/info/output.html#d:cmapx>.
+
+Their sample code has a dot file - x.gv - containing this line:
+
+	URL="http://www.research.att.com/base.html";
+
+The way you set such a url in C<GraphViz2> is via a new parameter to C<new()>. This parameter is called C<im_meta>
+and it takes a hashref as a value. Currently the only key used within that hashref is the case-sensitive C<URL>.
+
+Thus you must do this to set a URL:
+
+	my($graph) = GraphViz2 -> new
+	             (
+	                ...
+	                im_meta =>
+	                {
+	                    URL => 'http://savage.net.au/maps/demo.3.1.html', # Note: URL must be in caps.
+	                },
+	             );
+
+See maps/demo.3.pl and maps/demo.4.pl for sample code.
+
+=head2 Typical Code
+
+Normally you would call C<run()> as:
+
+	$graph -> run
+	(
+	    format      => $format,
+	    output_file => $output_file
+	);
+
+That line was copied from scripts/cluster.pl.
+
+To trigger image map processing, you must include 2 new parameters:
+
+	$graph -> run
+	(
+	    format         => $format,
+	    output_file    => $output_file,
+	    im_format      => $im_format,
+	    im_output_file => $im_output_file
+	);
+
+That line was copied from maps/demo.3.pl, and there is an identical line in maps/demo.4.pl.
+
+=head2 The New Parameters to run()
+
+=over 4
+
+=item o im_format => $str
+
+Expected values: 'imap' (server-side) and 'cmapx' (client-side).
+
+Default value: 'cmapx'.
+
+=item o im_output_file => $file_name
+
+The name of the output map file.
+
+Default: ''.
+
+=back
+
+=head2 Sample Code
+
+Various demos are shipped in the new maps/ directory:
+
+Each demo, when FTPed to your web server displays some text with an image in the middle. In each case
+you can click on the upper oval to jump to one page, or click on the lower oval to jump to a different
+page, or click anywhere else in the image to jump to a third page.
+
+=over 4
+
+=item o demo.1.*
+
+This set demonstrates a server-side image map but does not use C<GraphViz2>.
+
+You have to run demo.1.sh which generates demo.1.map, and then you FTP the whole dir maps/ to your web server.
+
+URL: your.domain.name/maps/demo.1.html.
+
+=item o demo.2.*
+
+This set demonstrates a client-side image map but does not use C<GraphViz2>.
+
+You have to run demo.2.sh which generates demo.2.map, and then you manually copy demo.2.map into demo.2.html,
+replacing any version of the map already present. After that you FTP the whole dir maps/ to your web server.
+
+URL: your.domain.name/maps/demo.2.html.
+
+=item o demo.3.*
+
+This set demonstrates a server-side image map using C<GraphViz2> via demo.3.pl.
+
+Note line 54 of demo.3.pl which sets the default C<im_format> to 'imap'.
+
+URL: your.domain.name/maps/demo.3.html.
+
+=item o demo.4.*
+
+This set demonstrates a client-side image map using C<GraphViz2> via demo.4.pl.
+
+As with demo.2.* there is some manually editing to be done.
+
+Note line 54 of demo.4.pl which sets the default C<im_format> to 'cmapx'. This is the only important
+difference between this demo and the previous one.
+
+There are other minor differences, in that one uses 'svg' and the other 'png'. And of course the urls
+of the web pages embedded in the code and in those web pages differs, just to demonstate that the maps
+do indeed lead to different pages.
+
+URL: your.domain.name/maps/demo.4.html.
+
+=back
+
 =head1 Methods
 
 =head2 add_edge(from => $from_node_name, to => $to_node_name, [label => $label, %hash])
@@ -1946,6 +2175,15 @@ See L</scripts/anonymous.pl> for demo code.
 
 If there is some specific requirement which this does not cater for, let me know and I can change the code.
 
+=head2 How do I use image maps?
+
+See L</Image Maps> above.
+
+=head2 I'm trying to use image maps but the non-image map code runs instead!
+
+The default value of C<im_output_file> is '', so if you do not set it to anything, the new image maps code
+is ignored.
+
 =head2 Why such a different approach to logging?
 
 As you can see from scripts/*.pl, I always use L<Log::Handler>.
@@ -2629,6 +2867,7 @@ z => node
 directed
 driver
 format
+im_format
 label
 name
 record_shape
@@ -2636,6 +2875,9 @@ strict
 subgraph
 timeout
 
+@@ im_meta
+URL
+
 @@ node
 Mcircle
 Mdiamond
